/*-------------------------------------------------------------------- +
|
|   BoxDependency.h
|
+--------------------------------------------------------------------- */
/*-------------------------------------------------------------------- +
|																		|
| Copyright (C) 2017 LA Solutions Ltd, 									|
| All rights reserved.													|
|																		|
+----------------------------------------------------------------------*/
#if !defined(BOX_DEPENDENCY_H_INCLUDED_)
#define BOX_DEPENDENCY_H_INCLUDED_
#pragma once

#include    <DgnPlatform/DgnPlatformBaseType.r.h>
#include    <DgnPlatform/DgnPlatform.h>
#include    <DgnPlatform/DependencyManagerLinkage.h>
#include    <DgnPlatform/DependencyManager.h>
#ifndef BENTLEY
#define BENTLEY
#endif
#include	<std_collection_typedefs.h>

///////////////////////////////////////////////////////////////////////
///	<summary>Dependency callback function.
///	<remarks>This callback is established by mdlDependency_registerCallback(), typically in MdlMain().</remarks>
///	</summary>
int					processingCallback
(				    // <= SUCCESS if callback succeeds; ERROR to undo all changes
Bentley::DgnPlatform::ElementId	    	dependentID,	// => element ID of dependent element
DgnModelRefP	    modelRef,    	// => file of dependent element
Bentley::DgnPlatform::DependencyLinkage*	pLinkage,  	// => copy of dependent element's linkage
UChar*				pRootStatus,	// => change status of each root in linkage: DEPENDENCY_STATUS_XXX
UChar		    	selfStatus
);
///////////////////////////////////////////////////////////////////////
///	<summary>
///	Determine whether a selection set is active.
///	<remarks>Issues a warning if there is no selection set.</remarks>
///	</summary>
bool								IsSelectionSetActive ();
///////////////////////////////////////////////////////////////////////
///	<summary>
///	Find an element from its ID and compute its origin.
///	</summary>
///	<remarks>
///	'Origin' is a nebulous concept.
///	</remarks>
///	<param name="origin" >The 'origin' of the element.</param>
///	<param name="origin" >The ID of the element to examine.</param>
///	<param name="model" >The model that stores this element.  Defaults to the active model.</param>
bool								GetElementOriginById
(
DPoint3d&							origin,
Bentley::DgnPlatform::ElementId		id,
DgnModelRefP						model = ACTIVEMODEL
);
///////////////////////////////////////////////////////////////////////
///	<summary>
///	Compute an offset value and write it into a linkage.
///	</summary>
void					replaceOffsetDependencyLinkage
(
MSElementDescrH			ppDependentElm,		//	<=>	Dependent element
UInt32*					filePos,			//	=>	Element original filePos
Bentley::DgnPlatform::DependencyLinkage*		pLinkage,			//	=>	Linkage data
size_t					size				//	=>	Linkage extra data size
);
///////////////////////////////////////////////////////////////////////
///	<summary>
///	Initialise a DRange3d from low and high values using the MicroStationAPI.
///	</summary>
void 					FindHighLow
(
DRange3d&				range,
DPoint3d const&			low,
DPoint3d const&			high
);
///////////////////////////////////////////////////////////////////////
///	<summary>
///	Initialise a DRange3d from low and high values using V8i techniques.
///	</summary>
void 					FindHighLow
(
DPoint3d&				low,
DPoint3d&				high,
DRange3d const&			range
);
///////////////////////////////////////////////////////////////////////
///	<summary>
///	Compute a range of points that forma a rectange about low, high.
///	</summary>
///	<param name="range">Array of boundary points, already dimensioned (i.e. DPoint3d range [5]).</param>
///	<param name="low">Lowest point to consider.</param>
///	<param name="high">Highest point to consider.</param>
void 					MakeShapeBoundary
(
DPoint3d*				range,
DPoint3d const&			low,
DPoint3d const&			high
);
///////////////////////////////////////////////////////////////////////
///	<summary>
///	Compute a range of points that forma a rectange about low, high.
///	</summary>
///	<param name="range">List of boundary points.</param>
///	<param name="low">Lowest point to consider.</param>
///	<param name="high">Highest point to consider.</param>
void 					MakeShapeBoundary
(
BentleyVectorDPoint3dR	range,
DPoint3dCR				low,
DPoint3dCR				high
);

#endif	// !defined(BOX_DEPENDENCY_H_INCLUDED_)
