/*----------------------------------------------------------------------+
|																		|
| MessageTracer Copyright 2010..2017, LA Solutions Ltd					|
| All rights reserved.													|
|																		|
| Limited permission is hereby granted to reproduce and modify this		|
| copyrighted material provided that this notice is retained			|
| in its entirety in any such reproduction or modification.				|
|																		|
+----------------------------------------------------------------------*/
#if !defined(MESSAGE_TRACER_H_INCLUDED_)
#define MESSAGE_TRACER_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
/*----------------------------------------------------------------------+
|                                                                       |
|   MessageTracer - MessageTracer source code. 							|
|                                                                       |
+----------------------------------------------------------------------*/
#include	<Mstn/MdlApi/msoutput.fdf>
#include	<DgnPlatform/DgnPlatformBaseType.r.h>
#include 	<string>

class CMessageTracer
{
	std::wstring	terse_;
	std::wstring	verbose_;

	static std::wstring	Convert			(std::string const&	s);
	static std::wstring	Convert			(char const* 		s);

public:
	///////////////////////////////////////////////////////////////////
	//	Implementation
	///////////////////////////////////////////////////////////////////
	//	Send a message to MicroStation's message center. By default,
	//	the message is shown with DEBUG priority (user must turn on
	//	debug messages in the message center.
	void			Output		(DgnPlatform::OutputMessagePriority	priority	= DgnPlatform::OutputMessagePriority::Debug)	const;
	void			Warning		(DgnPlatform::OutputMessagePriority	priority	= DgnPlatform::OutputMessagePriority::Warning,
								bool								pop			= true)	const;
	///////////////////////////////////////////////////////////////////
	//	Provide an alternative message to the current message
	void			Message		(const char*			msg);
	void			Message		(const std::string&		msg);
	///////////////////////////////////////////////////////////////////
	//	Provide a verbose message that differs from the terse message
	void			Verbose		(const char*			msg);
	void			Verbose		(const std::string&		msg);
	///////////////////////////////////////////////////////////////////
	//	Append new text to the current message
	void			Space		(int					count	= 1);
	void			Quote		(const char*			suffix);
	void			Quote		(const std::string&		suffix);
	void			Quote		(const WChar*			suffix);
	void			Quote		(const std::wstring&	suffix);
	void			Append		(const char*			suffix);
	void			Append		(const std::string&		suffix);
	void			Append		(const wchar_t*			suffix);
	void			Append		(const std::wstring&	suffix);
	void			Append		(bool					state);
	void			Append		(UInt16					number);
	void			Append		(int					number);
	void			Append		(long					number);
	void			Append		(UInt32					number);
	void			Append		(size_t					number);
	void			ElementId	(Bentley::DgnPlatform::ElementId const& id);
	///	<summary>UInt64 is Bentley::DgnPlatform::ElementId.</summary>
	void			Append		(UInt64 const&			id);
	void			Append		(DPoint3d const&		pt,
								DgnModelRefP			modelRef	= ACTIVEMODEL,
								int						precision	= 2);
	///	<remarks>Vector method distinguishes DVec3d from DPoint3d</remarks>
	void			Vector		(DVec3d const&			v,
								int						precision	= 2);
	void			Append		(double					d,
								DgnModelRefP			modelRef	= ACTIVEMODEL,
								int						precision	= 2);
	///	<remarks>Default precision of 7 matches Analyse Element command output</remarks>
	void			Append		(RotMatrix const&		r,
								int						precision	= 7);
	void			IdentifyElm	(MSElementDescrCP		el);
	void			Degrees		(double					radians);
	void			Radians		(double					radians);
	void			Euler		(RotMatrix const&		r,
								int						precision	= 7);
	///////////////////////////////////////////////////////////////////
	//	operators
	CMessageTracer&	operator =	(const char*				msg);
	CMessageTracer&	operator =	(const std::string&			msg);
	CMessageTracer&	operator =	(const WChar*				msg);
	CMessageTracer&	operator =	(const std::wstring&		msg);
	CMessageTracer&	operator =	(const Bentley::WString&	msg);
	CMessageTracer&	operator +=	(const char*				msg);
	CMessageTracer&	operator +=	(const std::string&			msg);
	CMessageTracer&	operator +=	(const WChar*				msg);
	CMessageTracer&	operator +=	(const std::wstring&		msg);
	CMessageTracer&	operator +=	(const Bentley::WString&	msg);

	///////////////////////////////////////////////////////////////////
	//	Construction
	CMessageTracer	(const char*			msg);
	CMessageTracer	(const std::string&		msg);
	CMessageTracer	(const WChar*			msg);
	CMessageTracer	(const std::wstring&	msg);
};

#endif // !defined(MESSAGE_TRACER_H_INCLUDED_)
