
#include    <Mstn/MdlApi/MdlApi.h>
#include 	<DgnView/DgnElementSetTool.h>
#include 	"BoxDependency.h"
#include 	"BoxDependencyLinkage.h"
#include 	"MessageTracer.h"
#include 	"PlaceCellTool.h"
#include 	<DgnPlatform/AssociativePoint.h>

USING_NAMESPACE_BENTLEY_DGNPLATFORM;

///////////////////////////////////////////////////////////////////////
bool PlaceCellTool::CreateElement (MSElementDescrP* cell, DPoint3dCR origin, Viewport const& view)
{
	const bool			UpdateMasterFile	{ true };
	//const bool			RelativeMode		{ false };
	const bool			TrueScale			{ false };
	const int			Unshared			{ 0 };
	const DPoint3dCP	NotScaled			{ nullptr };
	short*				NoAttributes		{ nullptr };
	const int			NoGraphicGroup		{ 0 };
	Bentley::RotMatrix	rotation;
	rotation.InitIdentity ();
	//RotMatrix			rotation			{ *view.GetRotMatrix () };
	//	Place cell adds the cell to the active model, which we don't want for dynamics
	//UInt32		filePos	= mdlCell_placeCell (&origin, NotScaled, TrueScale, &rotation, NoAttributes, NoGraphicGroup, RelativeMode, NoGraphicGroup, Unshared, cellName_.c_str (), nullptr);
	bool				rc					{ false };
	if (SUCCESS == mdlCell_getElmDscr (cell, nullptr, &origin, NotScaled, TrueScale, &rotation, NoAttributes, NoGraphicGroup, Unshared, UpdateMasterFile, cellName_.c_str (), nullptr))
	{
		rc	= true;
	}
    return rc;
}
///////////////////////////////////////////////////////////////////////
void PlaceCellTool::_OnDynamicFrame (DgnButtonEventCR ev)
{
	MSElementDescrP		cell			{ nullptr };
    if (!CreateElement (&cell, *ev.GetPoint (), *ev.GetViewport ()))
        return;
    const bool&			TakeOwnership	{ true };
    const bool&			NotInCache		{ false };
	EditElementHandle	eeh (cell, TakeOwnership, NotInCache, ACTIVEMODEL);

    RedrawElems 		redrawElems;
    // Display in all views, draws to cursor view first...
    redrawElems.SetDynamicsViews (IViewManager::GetActiveViewSet (), ev.GetViewport ());
    redrawElems.SetDrawMode (DRAW_MODE_TempDraw);
    redrawElems.SetDrawPurpose (DrawPurpose::Dynamics);
    redrawElems.DoRedraw (eeh);
}
///////////////////////////////////////////////////////////////////////
bool PlaceCellTool::_OnDataButton (DgnButtonEventCR ev)
{
	//Create cell element and add it to model.
	MSElementDescrP		cell			{ nullptr };
    if (CreateElement (&cell, *ev.GetPoint (), *ev.GetViewport ()))
	{
		const bool&		TakeOwnership	{ true };
		const bool&		NotInCache		{ false };
		EditElementHandle	eeh (cell, TakeOwnership, NotInCache, ACTIVEMODEL);
		AddLinkage (eeh, *ev.GetPoint (), dependencyType_);
		if (SUCCESS == eeh.AddToModel ())
		{
			CMessageTracer msg (L"_OnDataButton: successfully addded linkage");
			msg.Output ();
		}
		else
		{
			WCharCP		msg	{ L"eeh.AddToModel() failed" };
			mdlOutput_messageCenter (DgnPlatform::OutputMessagePriority::Warning, msg, msg, DgnPlatform::OutputMessageAlert::Balloon);
		}
	}
	return _CheckSingleShot ();
}
///////////////////////////////////////////////////////////////////////
/*
bool				PlaceCellTool::GetElementOriginById
(
DPoint3d&			origin,
ElementId			id
)
{
	bool			rc				{ false };
	ElementHandle	eh				{ id,  ACTIVEMODEL };
	DisplayHandlerP	handler			{ eh.GetDisplayHandler () };
	if (nullptr != handler)
	{
		handler->GetSnapOrigin (eh, origin);	//	Alternative is GetTransformOrigin()
		rc	= true;
	}
	return rc;
}
*/
///////////////////////////////////////////////////////////////////////
bool					PlaceCellTool::CreateOffsetIdDependencyLinkage
(
EditElementHandleR		eeh,
DPoint3dCR				origin,
DPoint3dCR 				placed
)
{
	CMessageTracer		msg (L"CreateOffsetIdDependencyLinkage root ID ");  msg.ElementId (rootId_);
	msg.Output ();
	bool				rc			{ false };
    OffsetIdDependencyLinkage	linkage;
    msg	= L"OffsetIdDependencyLinkage.read<DEPENDENCY_DATA_TYPE_ELEM_ID>="; msg.Append (linkage.read2<DEPENDENCY_DATA_TYPE_ELEM_ID>());
	msg.Output ();
    msg	= L"OffsetIdDependencyLinkage.read3="; msg.Append (linkage.read3());
	msg.Output ();
	linkage.AddRootId (rootId_);
    //	Add extra data - must be correctly aligned and linkage
    //	buffer declared to be of sufficient size to hold both roots
    //	and extra data
	DVec3d				offset;
	offset.DifferenceOf (placed, origin);
	linkage.SetOffset (offset);
	rc	= linkage.AppendLinkage (eeh);
	if (rc)
	{
		msg	= L"Computed offset "; msg.Append (offset);
		msg += L" stored offset "; msg.Append (linkage.GetOffset ());
		msg.Output ();
	}
	else
	{
		WCharCP		msg	{ L"DependencyManagerLinkage::AppendLinkage() failed" };
		mdlOutput_messageCenter (DgnPlatform::OutputMessagePriority::Warning, msg, msg, DgnPlatform::OutputMessageAlert::Balloon);
	}
	return rc;
}
///////////////////////////////////////////////////////////////////////
bool					PlaceCellTool::CreateOffsetAssocDependencyLinkage
(
EditElementHandleR		eeh,
DPoint3dCR				origin,
DPoint3dCR 				placed,
AssocPointCR			assocPoint
)
{
	CMessageTracer		msg (L"CreateOffsetAssocDependencyLinkage root ID ");  msg.ElementId (rootId_);
	msg.Output ();
	bool				rc			{ false };
    OffsetAssocDependencyLinkage	linkage;
    //msg	= L"OffsetAssocDependencyLinkage.trivial="; msg.Append (linkage.trivial());
	//msg.Output ();
    msg	= L"OffsetAssocDependencyLinkage.read<DEPENDENCY_DATA_TYPE_ASSOC_POINT>="; msg.Append (linkage.read2<DEPENDENCY_DATA_TYPE_ASSOC_POINT>());
	msg.Output ();
    msg	= L"OffsetAssocDependencyLinkage.read3="; msg.Append (linkage.read3());
	msg.Output ();
    linkage.AddRootAssoc (assocPoint);
    //	Add extra data - must be correctly aligned and linkage
    //	buffer declared to be of sufficient size to hold both roots
    //	and extra data
	DVec3d				offset;
	offset.DifferenceOf (placed, origin);
	linkage.SetOffset (offset);
	rc	= linkage.AppendLinkage (eeh);
	if (rc)
	{
		//msg	= L"Computed offset "; msg.Append (offset);
		//msg += L" stored offset "; msg.Append (linkage.GetOffset ());
		//msg.Output ();
	}
	else
	{
		WCharCP		msg	{ L"DependencyManagerLinkage::AppendLinkage() failed" };
		mdlOutput_messageCenter (DgnPlatform::OutputMessagePriority::Warning, msg, msg, DgnPlatform::OutputMessageAlert::Balloon);
	}
	return rc;
}
///////////////////////////////////////////////////////////////////////
bool	PlaceCellTool::AddLinkage		(EditElementHandleR	eeh, DPoint3dCR	point, AppValues dependencyType)
{
	bool			rc		{ false };
	//	Compute 'origin' of root element having rootId_
	DPoint3d		origin;
	GetElementOriginById (origin, rootId_);
	switch (dependencyType)
	{
		case TYPE_XYOFFSET_ID:
			//	Create dependency linkage for rootId_ including offset from origin
			rc	= CreateOffsetIdDependencyLinkage (eeh, origin, point);
		break;
		case TYPE_XYOFFSET_ASSOC:
		{
			DgnPlatform::AssocPoint	assocPoint;
			CreateAssocPointFromRoot (assocPoint, rootId_);
			//	Create dependency linkage for rootId_ including assoc point on root and offset from origin
			rc	= CreateOffsetAssocDependencyLinkage (eeh, origin, point, assocPoint);
		}
		break;
	}
	return rc;
}
///////////////////////////////////////////////////////////////////////
void	PlaceCellTool::CreateAssocPointFromRoot (AssocPoint& assocPoint, Bentley::DgnPlatform::ElementId	rootID)
{
	ElementHandle	eh	(rootID, ACTIVEMODEL);
	AssociativePoint::InitOrigin (assocPoint, 0);
	AssociativePoint::SetRoot (assocPoint, rootID, 0);
}
///////////////////////////////////////////////////////////////////////
void 	PlaceCellTool::_OnPostInstall ()
{
    //Enable snapping for create tools.
    AccuSnap::GetInstance ().EnableSnap (true);

    //Start dynamics as soon as the tool starts.
    _BeginDynamics ();

    __super::_OnPostInstall ();
}
///////////////////////////////////////////////////////////////////////
void PlaceCellTool::SetCellName(WCharCP text)
{
	cellName_ = text;
}
///////////////////////////////////////////////////////////////////////
void PlaceCellTool::SetRootId(ElementId id)
{
	rootId_ = id;
}
///////////////////////////////////////////////////////////////////////
void PlaceCellTool::SetDependencyType(AppValues dependencyType)
{
	dependencyType_ = dependencyType;
}
///////////////////////////////////////////////////////////////////////
void PlaceCellTool::InstallNewInstance (int toolId, int toolPrompt, WCharCP cellName, ElementId	rootID, AppValues dependencyType)
{
    PlaceCellTool* tool = new PlaceCellTool (toolId, toolPrompt);
    tool->SetCellName(cellName);
    tool->SetRootId (rootID);
    tool->SetDependencyType (dependencyType);
    tool->InstallTool ();
}
///////////////////////////////////////////////////////////////////////


