
#if !defined(PLACE_CELL_TOOL_H_INCLUDED_)
#define PLACE_CELL_TOOL_H_INCLUDED_
#pragma once

#include 	<DgnPlatform/DgnPlatformAPI.h>
#include 	<DgnPlatform/TextHandlers.h>
#include 	<DgnView/DgnTool.h>
#include 	<Mstn/isessionmgr.h>
#include 	<Mstn/ElementPropertyUtils.h>
#include 	"BoxDependencyIds.h"

///////////////////////////////////////////////////////////////////////
///	<summary>PlaceCellTool class</summary>
struct PlaceCellTool : Bentley::DgnPlatform::DgnPrimitiveTool
{

private:
	///	<summary>Name of a cell that we place as an example dependent element.</summary>
    WString             			cellName_;
	///	<summary>Element ID of the root element on which the newly-placed cell is dependent.</summary>
	Bentley::DgnPlatform::ElementId	rootId_;
	///	<summary>The type of dependency we will create.
	///	<remarks>Several types of dependency are possible.
	///	We have a working offset dependency specified by a root element ID.
	///	<para>An alternative is to try an associative point dependency,
	///	which is currently unproven.</para>
	///	<para>See BoxDependencyIds.h.</para>
	///	</remarks>
	///	</summary>
	AppValues						dependencyType_;

protected:

///	<summary>PlaceCellTool inherits from DgnPrimitiveTool.</summary>
PlaceCellTool (int toolName, int toolPrompt)
	: 	Bentley::DgnPlatform::DgnPrimitiveTool (toolName, toolPrompt),
		cellName_		(L""),
		rootId_			(0),
		dependencyType_	(TYPE_XYOFFSET_ID)
{}

///	<summary>_OnPostInstall.</summary>
virtual void _OnPostInstall() override;

///	<summary>_OnRestartTool.</summary>
virtual void _OnRestartTool() override { InstallNewInstance (GetToolId (), GetToolPrompt (), cellName_.c_str (),
			rootId_, dependencyType_); }

///	<summary>_OnDataButton.</summary>
virtual bool _OnDataButton(DgnButtonEventCR ev) override;

///	<summary>_OnResetButton.</summary>
virtual bool _OnResetButton(DgnButtonEventCR ev) override { _ExitTool(); return true; }

///	<summary>_OnDynamicFrame.</summary>
virtual void _OnDynamicFrame(DgnButtonEventCR ev) override;

///	<summary>CreateElement.</summary>
///	<remarks>We want an in-memory element during dynamics.</remarks>
///	<returns>EditElementHandle if element successfully created.</returns>
bool	CreateElement	(MSElementDescrP* cell, DPoint3dCR origin, Bentley::DgnPlatform::Viewport const& view);
///	<summary>Add dependency linkage to newly-placed cell.</summary>
bool	AddLinkage		(EditElementHandleR	eeh, DPoint3dCR	point, AppValues dependencyType);
///	<summary>Find the 'origin' of an element.</summary>
///	<remarks>Note that the DisplayHandler provides methods to get an element 'origin'.
///	e.g. GetSnapOrigin, GetTransformOrigin</remarks>
///	<param name="origin">Origin point.</param>
///	<param name="id">Element ID</param>
/*
bool	GetElementOriginById
(
DPoint3d&							origin,
Bentley::DgnPlatform::ElementId		id
);
*/
///	<summary>CreateOffsetIdDependencyLinkage.</summary>
///	<param name="origin">Root element origin point.</param>
///	<param name="placed">Element placement point</param>
bool							CreateOffsetIdDependencyLinkage
(
EditElementHandleR				eeh,
DPoint3dCR						origin,
DPoint3dCR 						placed
);
///	<summary>CreateOffsetAssocDependencyLinkage.</summary>
///	<param name="origin">Root element origin point.</param>
///	<param name="placed">Element placement point</param>
bool							CreateOffsetAssocDependencyLinkage
(
EditElementHandleR				eeh,
DPoint3dCR						origin,
DPoint3dCR 						placed,
AssocPointCR					assocPoint
);
///	<summary>SetText.</summary>
void SetCellName		(WCharCP text);
///	<summary>SetRootId.
///	<remarks>The ID of the element on which this cell will be dependent.</remarks>
///	</summary>
void SetRootId			(Bentley::DgnPlatform::ElementId	rootID);
///	<summary>SetDependencyType.
///	<remarks>Specify the kind of dependency we want to create.</remarks>
///	</summary>
void SetDependencyType	(AppValues	dependencyType);
///	<summary>CreateAssocPointFromRoot.
///	<remarks>Get the root element and create an association point.</remarks>
///	</summary>
void CreateAssocPointFromRoot (DgnPlatform::AssocPoint& assocPoint, Bentley::DgnPlatform::ElementId	rootID);

public:

	///	<summary>InstallNewInstance.</summary>
	///	<remarks>static function.</remarks>
	static void InstallNewInstance(int toolId, int toolPrompt, WCharCP cellName,
		Bentley::DgnPlatform::ElementId	rootID, AppValues dependencyType);

}; // PlaceCellTool

#endif	// !defined(PLACE_CELL_TOOL_H_INCLUDED_)
